@@ -19,6 +19,11 @@
 
 #include "win_utils.h"
 #include "parse_cmdline.h"
+#include "version.h"
+
+/*=========================================================================
+ * String functions
+ *=========================================================================*/
 
 /**
  * Print a 0-terminated string representation of a 64-bit number to
@@ -34,15 +39,15 @@ void sprintI64(char *dst, uint64_t number, int min_width)
 	size_t len;
 	char *p = buf + 23; /* start filling from the buffer end */
 	*(p--) = 0; /* last symbol should be '\0' */
-	if(number == 0) {
+	if (number == 0) {
 		*(p--) = '0';
 	} else {
-		for(; p >= buf && number != 0; p--, number /= 10) {
+		for (; p >= buf && number != 0; p--, number /= 10) {
 			*p = '0' + (char)(number % 10);
 		}
 	}
 	len = buf + 22 - p;
-	if((size_t)min_width > len) {
+	if ((size_t)min_width > len) {
 		memset(dst, 0x20, min_width - len); /* fill by spaces */
 		dst += min_width - len;
 	}
@@ -58,7 +63,7 @@ void sprintI64(char *dst, uint64_t number, int min_width)
 int int_len(uint64_t num)
 {
 	int len;
-	for(len = 0; num; len++, num /= 10);
+	for (len = 0; num; len++, num /= 10);
 	return (len == 0 ? 1 : len); /* note: int_len(0) == 1 */
 }
 
@@ -95,14 +100,14 @@ static char* print_hex_byte(char *dst, const unsigned char byte, int upper_case)
 int urlencode(char *dst, const char *name)
 {
 	const char *start;
-	if(!dst) {
+	if (!dst) {
 		int len;
-		for(len = 0; *name; name++) len += (IS_GOOD_URL_CHAR(*name) ? 1 : 3);
+		for (len = 0; *name; name++) len += (IS_GOOD_URL_CHAR(*name) ? 1 : 3);
 		return len;
 	}
 	/* encode URL as specified by RFC 1738 */
-	for(start = dst; *name; name++) {
-		if( IS_GOOD_URL_CHAR(*name) ) {
+	for (start = dst; *name; name++) {
+		if ( IS_GOOD_URL_CHAR(*name) ) {
 			*dst++ = *name;
 		} else {
 			*dst++ = '%';
@@ -125,8 +130,8 @@ char* str_tolower(const char* str)
 {
 	char* buf = rsh_strdup(str);
 	char* p;
-	if(buf) {
-		for(p = buf; *p; p++) *p = tolower(*p);
+	if (buf) {
+		for (p = buf; *p; p++) *p = tolower(*p);
 	}
 	return buf;
 }
@@ -140,8 +145,8 @@ char* str_tolower(const char* str)
 char* str_trim(char* str)
 {
 	char* last = str + strlen(str) - 1;
-	while(isspace((unsigned char)*str)) str++;
-	while(isspace((unsigned char)*last) && last > str) *(last--) = 0;
+	while (isspace((unsigned char)*str)) str++;
+	while (isspace((unsigned char)*last) && last > str) *(last--) = 0;
 	return str;
 }
 
@@ -189,8 +194,8 @@ char* str_append(const char* orig, const char* append)
  */
 int is_binary_string(const char* str)
 {
-	for(; *str; str++) {
-		if(((unsigned char)*str) < 32 && ((1 << (unsigned char)*str) & ~0x2600)) {
+	for (; *str; str++) {
+		if (((unsigned char)*str) < 32 && ((1 << (unsigned char)*str) & ~0x2600)) {
 			return 1;
 		}
 	}
@@ -206,24 +211,31 @@ int is_binary_string(const char* str)
 size_t strlen_utf8_c(const char *str)
 {
 	size_t length = 0;
-	for(; *str; str++) {
-		if((*str & 0xc0) != 0x80) length++;
+	for (; *str; str++) {
+		if ((*str & 0xc0) != 0x80) length++;
 	}
 	return length;
 }
 
-/**
-* Exit the program, with restoring console state.
-*
-* @param code the program exit code
-*/
-void rhash_exit(int code)
+/*=========================================================================
+* Program version information
+*=========================================================================*/
+
+const char* get_version_string(void)
 {
-	IF_WINDOWS(restore_console());
-	exit(code);
+	static const char* version_string = VERSION;
+	return version_string;
 }
 
-/* FILE FUNCTIONS */
+const char* get_bt_program_name(void)
+{
+	static const char* bt_program_name = PROGRAM_NAME "/" VERSION;
+	return bt_program_name;
+}
+
+/*=========================================================================
+ * Path functions
+ *=========================================================================*/
 
 /**
  * Return filename without path.
@@ -234,7 +246,7 @@ void rhash_exit(int code)
 const char* get_basename(const char* path)
 {
 	const char *p = path + strlen(path) - 1;
-	for(; p >= path && !IS_PATH_SEPARATOR(*p); p--);
+	for (; p >= path && !IS_PATH_SEPARATOR(*p); p--);
 	return (p+1);
 }
 
@@ -249,8 +261,8 @@ char* get_dirname(const char* path)
 {
 	const char *p = path + strlen(path) - 1;
 	char *res;
-	for(; p > path && !IS_PATH_SEPARATOR(*p); p--);
-	if((p - path) > 1) {
+	for (; p > path && !IS_PATH_SEPARATOR(*p); p--);
+	if ((p - path) > 1) {
 		res = (char*)rsh_malloc(p-path+1);
 		memcpy(res, path, p-path);
 		res[p-path] = 0;
@@ -275,7 +287,12 @@ char* make_path(const char* dir_path, const char* filename)
 	assert(filename);
 
 	/* remove leading path separators from filename */
-	while(IS_PATH_SEPARATOR(*filename)) filename++;
+	while (IS_PATH_SEPARATOR(*filename)) filename++;
+
+	if (dir_path[0] == '.' && dir_path[1] == 0) {
+		/* do not extend filename for dir_path="." */
+		return rsh_strdup(filename);
+	}
 
 	/* copy directory path */
 	len = strlen(dir_path);
@@ -283,7 +300,7 @@ char* make_path(const char* dir_path, const char* filename)
 	strcpy(buf, dir_path);
 
 	/* separate directory from filename */
-	if(len > 0 && !IS_PATH_SEPARATOR(buf[len-1])) {
+	if (len > 0 && !IS_PATH_SEPARATOR(buf[len-1])) {
 		buf[len++] = SYS_PATH_SEPARATOR;
 	}
 
@@ -319,38 +336,54 @@ int are_paths_equal(const rsh_tchar* a, const rsh_tchar* b)
 	return (*a == *b);
 }
 
-/**
- * Print time formated as hh:mm.ss YYYY-MM-DD to a file stream.
- *
- * @param out the stream to print time to
- * @param time the time to print
- */
-void print_time(FILE *out, time_t time)
-{
-	struct tm *t = localtime(&time);
-	static struct tm zero_tm;
-	if(t == NULL) {
-		/* if strange day, then print `00:00.00 1900-01-00' */
-		t = &zero_tm;
-		t->tm_hour = t->tm_min = t->tm_sec =
-		t->tm_year = t->tm_mon = t->tm_mday = 0;
-	}
-	fprintf(out, "%02u:%02u.%02u %4u-%02u-%02u", t->tm_hour, t->tm_min,
-		t->tm_sec, (1900+t->tm_year), t->tm_mon+1, t->tm_mday);
-}
-
-void print_time64(FILE *out, uint64_t time)
-{
-	print_time(out, (time_t)time);
-}
+/*=========================================================================
+ * Timer functions
+ *=========================================================================*/
 
 /**
- * Return ticks in milliseconds for time intervals measurement.
- * This function should be not precise but the fastest one
- * to retrieve internal clock value.
+ * Return real-value representing number of seconds
+ * stored in the given timeval structure.
+ * The function is used with timers, when printing time statistics.
  *
- * @return ticks count in milliseconds
+ * @param delta time delta to be converted
+ * @return number of seconds
  */
+static double rsh_fsec(timedelta_t* timer)
+{
+#ifdef _WIN32
+	LARGE_INTEGER freq;
+	QueryPerformanceFrequency(&freq);
+	return (double)*timer / freq.QuadPart;
+#else
+	return ((double)timer->tv_usec / 1000000.0) + timer->tv_sec;
+#endif
+}
+
+#ifdef _WIN32
+#include <windows.h>
+#define get_timedelta(delta) QueryPerformanceCounter((LARGE_INTEGER*)delta)
+#else
+#define get_timedelta(delta) gettimeofday(delta, NULL)
+#endif
+
+void rsh_timer_start(timedelta_t* timer)
+{
+	get_timedelta(timer);
+}
+
+double rsh_timer_stop(timedelta_t* timer)
+{
+	timedelta_t end;
+	get_timedelta(&end);
+#ifdef _WIN32
+	*timer = end - *timer;
+#else
+	timer->tv_sec  = end.tv_sec  - timer->tv_sec - (end.tv_usec >= timer->tv_usec ? 0 : 1);
+	timer->tv_usec = end.tv_usec + (end.tv_usec >= timer->tv_usec ? 0 : 1000000 ) - timer->tv_usec;
+#endif
+	return rsh_fsec(timer);
+}
+
 unsigned rhash_get_ticks(void)
 {
 #ifdef _WIN32
@@ -362,6 +395,39 @@ unsigned rhash_get_ticks(void)
 #endif
 }
 
+/*=========================================================================
+ * File functions
+ *=========================================================================*/
+
+void rsh_file_init(file_t* file, const char* path, int reuse_path)
+{
+	memset(file, 0, sizeof(*file));
+	if (reuse_path)
+	{
+		file->path = (char*)path;
+		file->mode = FILE_OPT_DONT_FREE_PATH;
+	} else {
+		file->path = rsh_strdup(path);
+	}
+}
+
+void rsh_file_cleanup(file_t* file)
+{
+	if ((file->mode & FILE_OPT_DONT_FREE_PATH) == 0)
+	{
+		free(file->path);
+	}
+	file->path = NULL;
+
+#ifdef _WIN32
+	free(file->wpath);
+	file->wpath = NULL;
+#endif /* _WIN32 */
+
+	file->mtime = file->size = 0;
+	file->mode = 0;
+}
+
 #ifdef _WIN32
 /**
  * Fill file information in the file_t structure.
@@ -374,10 +440,11 @@ int rsh_file_statw(file_t* file)
 	WIN32_FILE_ATTRIBUTE_DATA data;
 
 	/* read file attributes */
-	if(GetFileAttributesExW(file->wpath, GetFileExInfoStandard, &data)) {
+	if (GetFileAttributesExW(file->wpath, GetFileExInfoStandard, &data)) {
 		uint64_t u;
 		file->size  = (((uint64_t)data.nFileSizeHigh) << 32) + data.nFileSizeLow;
-		file->mode = (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? FILE_IFDIR : 0);
+		file->mode &= FILE_OPT_DONT_FREE_PATH;
+		file->mode |= (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? FILE_IFDIR : FILE_IFREG);
 
 		/* the number of 100-nanosecond intervals since January 1, 1601 */
 		u = (((uint64_t)data.ftLastWriteTime.dwHighDateTime) << 32) + data.ftLastWriteTime.dwLowDateTime;
@@ -404,14 +471,14 @@ int rsh_file_stat2(file_t* file, int use_lstat)
 	(void)use_lstat; /* ignore on windows */
 
 	file->mtime = 0;
-	if(file->wpath) {
+	if (file->wpath) {
 		free(file->wpath);
 		file->wpath = NULL;
 	}
 
-	for(i = 0; i < 2; i++) {
+	for (i = 0; i < 2; i++) {
 		file->wpath = c2w(file->path, i);
-		if(file->wpath == NULL) continue;
+		if (file->wpath == NULL) continue;
 
 		/* return on success */
 		if (rsh_file_statw(file) == 0) return 0;
@@ -423,21 +490,39 @@ int rsh_file_stat2(file_t* file, int use_lstat)
 	if (!errno) errno = EINVAL;
 	return -1;
 #else
-	struct rsh_stat_struct st;
-	int res = -1;
-	res = (use_lstat ? lstat(file->path, &st) : rsh_stat(file->path, &st));
+	struct stat st;
+	int res = 0;
+	file->mode &= FILE_OPT_DONT_FREE_PATH;
+
+	do {
+		if (use_lstat) {
+			/* check for symlink */
+			if (lstat(file->path, &st) < 0) return -1;
+			if (!S_ISLNK(st.st_mode)) break;
+
+			/* it's a symlink */
+			file->mode |= FILE_IFLNK;
+		}
+
+		res = stat(file->path, &st);
+	} while (0);
+
 	file->size  = st.st_size;
 	file->mtime = st.st_mtime;
 
-	file->mode  = 0;
-	if(S_ISDIR(st.st_mode)) file->mode |= FILE_IFDIR;
+	if (S_ISDIR(st.st_mode)) {
+		file->mode |= FILE_IFDIR;
+	} else if (S_ISREG(st.st_mode)) {
+		/* it's a regular file or a symlink pointing to a regular file */
+		file->mode |= FILE_IFREG;
+	}
 
 	return res;
 #endif /* _WIN32 */
 }
 
 /**
- * Fill file information in the file_t structure.
+ * Read the file information like its type, size and modification date.
  *
  * @param file the file information
  * @return 0 on success, -1 on error
@@ -447,21 +532,48 @@ int rsh_file_stat(file_t* file)
 	return rsh_file_stat2(file, 0);
 }
 
-void rsh_file_cleanup(file_t* file)
+int is_regular_file(const char* path)
 {
-	free(file->path);
-	file->path = NULL;
+	int is_regular = 0;
+	file_t file;
 
-#ifdef _WIN32
-	free(file->wpath);
-	file->wpath = NULL;
-#endif /* _WIN32 */
-
-	file->mtime = file->size = 0;
-	file->mode = 0;
+	rsh_file_init(&file, path, 1);
+	if (rsh_file_stat(&file) >= 0) {
+		is_regular = FILE_ISREG(&file);
+	}
+	rsh_file_cleanup(&file);
+	return is_regular;
 }
 
-/* program exit and error reporting functions */
+int if_file_exists(const char* path)
+{
+	int exists;
+	file_t file;
+
+	rsh_file_init(&file, path, 1);
+	exists = (rsh_file_stat(&file) >= 0);
+	rsh_file_cleanup(&file);
+	return exists;
+}
+
+/*=========================================================================
+ * Custom program exit function
+ *=========================================================================*/
+
+/**
+* Exit the program, with restoring console state.
+*
+* @param code the program exit code
+*/
+void rhash_exit(int code)
+{
+	IF_WINDOWS(restore_console());
+	exit(code);
+}
+
+/*=========================================================================
+ * Error reporting functions
+ *=========================================================================*/
 
 static void report_error_default(const char* srcfile, int srcline,
 	const char* format, ...);
@@ -486,7 +598,9 @@ static void report_error_default(const char* srcfile, int srcline, const char* f
 	va_end(ap);
 }
 
-/* MEMORY FUNCTIONS */
+/*=========================================================================
+ * Memory functions
+ *=========================================================================*/
 
 /**
  * Allocates a buffer via malloc with reporting memory error to stderr.
@@ -499,7 +613,7 @@ static void report_error_default(const char* srcfile, int srcline, const char* f
 void* rhash_malloc(size_t size, const char* srcfile, int srcline)
 {
 	void* res = malloc(size);
-	if(!res) {
+	if (!res) {
 		rsh_report_error(srcfile, srcline, "%s(%u) failed\n", "malloc", (unsigned)size);
 		rsh_exit(2);
 	}
@@ -518,7 +632,7 @@ void* rhash_malloc(size_t size, const char* srcfile, int srcline)
 void* rhash_calloc(size_t num, size_t size, const char* srcfile, int srcline)
 {
 	void* res = calloc(num, size);
-	if(!res) {
+	if (!res) {
 		rsh_report_error(srcfile, srcline, "calloc(%u, %u) failed\n", (unsigned)num, (unsigned)size);
 		rsh_exit(2);
 	}
@@ -540,10 +654,10 @@ char* rhash_strdup(const char* str, const char* srcfile, int srcline)
 	char* res = strdup(str);
 #else
 	char* res = (char*)malloc(strlen(str)+1);
-	if(res) strcpy(res, str);
+	if (res) strcpy(res, str);
 #endif
 
-	if(!res) {
+	if (!res) {
 		rsh_report_error(srcfile, srcline, "strdup(\"%s\") failed\n", str);
 		rsh_exit(2);
 	}
@@ -559,12 +673,17 @@ char* rhash_strdup(const char* str, const char* srcfile, int srcline)
  * @param srcline source code line to be reported on fail
  * @return allocated memory buffer with copied string
  */
-wchar_t* rhash_wcsdup(wchar_t* str, const char* srcfile, int srcline)
+wchar_t* rhash_wcsdup(const wchar_t* str, const char* srcfile, int srcline)
 {
+#ifndef __STRICT_ANSI__
 	wchar_t* res = wcsdup(str);
+#else
+	wchar_t* res = (wchar_t*)malloc((wcslen(str) + 1) * sizeof(wchar_t));
+	if (res) wcscpy(res, str);
+#endif
 
-	if(!res) {
-		rsh_report_error(srcfile, srcline, "wcsdup() failed\n");
+	if (!res) {
+		rsh_report_error(srcfile, srcline, "wcsdup(\"%u\") failed\n", (wcslen(str) + 1));
 		rsh_exit(2);
 	}
 	return res;
@@ -583,14 +702,16 @@ wchar_t* rhash_wcsdup(wchar_t* str, const char* srcfile, int srcline)
 void* rhash_realloc(void* mem, size_t size, const char* srcfile, int srcline)
 {
 	void* res = realloc(mem, size);
-	if(!res) {
+	if (!res) {
 		rsh_report_error(srcfile, srcline, "realloc(%p, %u) failed\n", mem, (unsigned)size);
 		rsh_exit(2);
 	}
 	return res;
 }
 
-/* vector functions */
+/*=========================================================================
+ * Containers
+ *=========================================================================*/
 
 /**
  * Allocate an empty vector.
@@ -626,10 +747,10 @@ struct vector_t* rsh_vector_new_simple(void)
  */
 void rsh_vector_destroy(vector_t* vect)
 {
-	if(!vect) return;
-	if(vect->destructor) {
+	if (!vect) return;
+	if (vect->destructor) {
 		unsigned i;
-		for(i=0; i<vect->size; i++) vect->destructor(vect->array[i]);
+		for (i=0; i<vect->size; i++) vect->destructor(vect->array[i]);
 	}
 	free(vect->array);
 	vect->size = vect->allocated = 0;
@@ -656,7 +777,7 @@ void rsh_vector_free(vector_t* vect)
 void rsh_vector_add_ptr(vector_t* vect, void* item)
 {
 	/* check if vect contains enough space for the next item */
-	if(vect->size >= vect->allocated) {
+	if (vect->size >= vect->allocated) {
 		size_t size = (vect->allocated==0 ? 128 : vect->allocated * 2);
 		vect->array = (void**)rsh_realloc(vect->array, size * sizeof(void*));
 		vect->allocated = size;
@@ -675,7 +796,7 @@ void rsh_vector_add_ptr(vector_t* vect, void* item)
 void rsh_vector_add_empty(struct vector_t* vect, size_t item_size)
 {
 	/* check if vect contains enough space for next item */
-	if(vect->size >= vect->allocated) {
+	if (vect->size >= vect->allocated) {
 		size_t size = (vect->allocated==0 ? 128 : vect->allocated * 2);
 		vect->array = (void**)rsh_realloc(vect->array, size * item_size);
 		vect->allocated = size;
@@ -705,7 +826,9 @@ void rsh_blocks_vector_destroy(blocks_vector_t* bvector)
 	rsh_vector_destroy(&bvector->blocks);
 }
 
-/* STRING BUFFER FUNCTIONS */
+/*=========================================================================
+ * String buffer functions
+ *=========================================================================*/
 
 /**
  * Allocate an empty string buffer.
@@ -726,7 +849,7 @@ strbuf_t* rsh_str_new(void)
  */
 void rsh_str_free(strbuf_t* ptr)
 {
-	if(ptr) {
+	if (ptr) {
 		free(ptr->str);
 		free(ptr);
 	}
@@ -741,8 +864,8 @@ void rsh_str_free(strbuf_t* ptr)
  */
 void rsh_str_ensure_size(strbuf_t *str, size_t new_size)
 {
-	if(new_size >= (size_t)str->allocated) {
-		if(new_size < 64) new_size = 64;
+	if (new_size >= (size_t)str->allocated) {
+		if (new_size < 64) new_size = 64;
 		str->str = (char*)rsh_realloc(str->str, new_size);
 		str->allocated = new_size;
 	}
