@@ -76,6 +76,10 @@ typedef  char rsh_tchar;
 # define to_utf8(str) NULL
 #endif /* _WIN32 */
 
+/* version information */
+const char* get_version_string(void);
+const char* get_bt_program_name(void);
+
 
 /**
  * Portable file information.
@@ -92,9 +96,14 @@ typedef struct file_t
 /* bit constants for the file_t.mode bit mask */
 #define FILE_IFDIR   0x01
 #define FILE_IFLNK   0x02
+#define FILE_IFREG   0x04
 #define FILE_IFROOT  0x10
 #define FILE_IFSTDIN 0x20
+#define FILE_OPT_DONT_FREE_PATH  0x40
+
 #define FILE_ISDIR(file) ((file)->mode & FILE_IFDIR)
+#define FILE_ISLNK(file) ((file)->mode & FILE_IFLNK)
+#define FILE_ISREG(file) ((file)->mode & FILE_IFREG)
 
 /* file functions */
 
@@ -103,11 +112,12 @@ char* get_dirname(const char* path);
 char* make_path(const char* dir, const char* filename);
 int are_paths_equal(const rsh_tchar* a, const rsh_tchar* b);
 
-void print_time(FILE *out, time_t time);
-void print_time64(FILE *out, uint64_t time);
+void rsh_file_init(file_t* file, const char* path, int reuse_path);
 void rsh_file_cleanup(file_t* file);
 int rsh_file_stat(file_t* file);
 int rsh_file_stat2(file_t* file, int use_lstat);
+int is_regular_file(const char* path);
+int if_file_exists(const char* path);
 
 #ifdef _WIN32
 # define get_file_tpath(file) ((file)->wpath)
@@ -119,24 +129,41 @@ int rsh_file_statw(file_t* file);
 #endif
 
 
-/* rhash stat function */
-#if (__MSVCRT_VERSION__ >= 0x0601) || (_MSC_VER >= 1400)
-# define rsh_stat_struct __stat64
-# define rsh_stat(path, st) win_stat(path, st)
-# define clib_wstat(path, st) _wstat64(path, st)
-#elif defined(_WIN32) && (defined(__MSVCRT__) || defined(_MSC_VER))
-# define rsh_stat_struct _stati64
-# define rsh_stat(path, st) win_stat(path, st)
-# define clib_wstat(path, st) _wstati64(path, st)
+/* time data and functions */
+
+/* portable timer definition */
+#ifdef _WIN32
+typedef unsigned long long timedelta_t;
 #else
-# define rsh_stat_struct stat
-# define rsh_stat(path, st) stat(path, st)
-/* # define clib_wstat(path, st) _wstat32(path, st) */
+#include <sys/time.h> /* for timeval */
+typedef struct timeval timedelta_t;
 #endif
 
-typedef struct rsh_stat_struct rsh_stat_buf;
+/**
+ * Start a timer.
+ *
+ * @param timer timer to start
+ */
+void rsh_timer_start(timedelta_t* timer);
 
+/**
+ * Stop given timer.
+ *
+ * @param timer the timer to stop
+ * @return number of seconds timed
+ */
+double rsh_timer_stop(timedelta_t* timer);
+
+/**
+ * Return ticks in milliseconds for time intervals measurement.
+ * This function should be not precise but the fastest one
+ * to retrieve internal clock value.
+ *
+ * @return ticks count in milliseconds
+ */
 unsigned rhash_get_ticks(void);
+
+
 void  rhash_exit(int code);
 
 
@@ -152,7 +179,7 @@ void* rhash_realloc(void* mem, size_t size, const char* srcfile, int srcline);
 
 #ifdef _WIN32
 #define rsh_wcsdup(str) rhash_wcsdup(str, __FILE__, __LINE__)
-wchar_t* rhash_wcsdup(wchar_t* str, const char* srcfile, int srcline);
+wchar_t* rhash_wcsdup(const wchar_t* str, const char* srcfile, int srcline);
 #endif
 
 
@@ -197,19 +224,19 @@ void rsh_blocks_vector_destroy(blocks_vector_t* vect);
 #define rsh_blocks_vector_get_ptr(bvector, index, blocksize, item_size) \
 	(&((unsigned char*)((bvector)->blocks.array[(index) / (blocksize)]))[(item_size) * ((index) % (blocksize))])
 #define rsh_blocks_vector_add(bvector, item, blocksize, item_size) { \
-	if(((bvector)->size % (blocksize)) == 0) \
+	if (((bvector)->size % (blocksize)) == 0) \
 	rsh_vector_add_ptr(&((bvector)->blocks), rsh_malloc((item_size) * (blocksize))); \
 	memcpy(rsh_blocks_vector_get_ptr((bvector), (bvector)->size, (blocksize), (item_size)), (item), (item_size)); \
 	(bvector)->size++; \
 }
 #define rsh_blocks_vector_add_ptr(bvector, ptr, blocksize) { \
-	if(((bvector)->size % (blocksize)) == 0) \
+	if (((bvector)->size % (blocksize)) == 0) \
 	rsh_vector_add_ptr(&((bvector)->blocks), rsh_malloc(sizeof(void*) * (blocksize))); \
 	((void***)(bvector)->blocks.array)[(bvector)->size / (blocksize)][(bvector)->size % (blocksize)] = (void*)ptr; \
 	(bvector)->size++; \
 }
 #define rsh_blocks_vector_add_empty(bvector, blocksize, item_size) { \
-	if( (((bvector)->size++) % (blocksize)) == 0) \
+	if ( (((bvector)->size++) % (blocksize)) == 0) \
 	rsh_vector_add_ptr(&((bvector)->blocks), rsh_malloc((item_size) * (blocksize))); \
 }
 
@@ -228,9 +255,9 @@ void rsh_str_append_n(strbuf_t *str, const char* text, size_t len);
 void rsh_str_append(strbuf_t *str, const char* text);
 
 #define rsh_str_ensure_length(str, len) \
-	if((size_t)(len) >= (size_t)(str)->allocated) rsh_str_ensure_size((str), (len) + 1);
+	if ((size_t)(len) >= (size_t)(str)->allocated) rsh_str_ensure_size((str), (len) + 1);
 #define rsh_wstr_ensure_length(str, len) \
-	if((size_t)((len) + sizeof(wchar_t)) > (size_t)(str)->allocated) rsh_str_ensure_size((str), (len) + sizeof(wchar_t));
+	if ((size_t)((len) + sizeof(wchar_t)) > (size_t)(str)->allocated) rsh_str_ensure_size((str), (len) + sizeof(wchar_t));
 
 #ifdef __cplusplus
 } /* extern "C" */
