@@ -49,7 +49,7 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 
 #ifdef _WIN32
 	/* expand wildcards and fill the root_files */
-	for(i = 0; i < count; i++)
+	for (i = 0; i < count; i++)
 	{
 		int added = 0;
 		size_t length, index;
@@ -57,7 +57,7 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 		wchar_t* p = wcschr(path, L'\0') - 1;
 
 		/* strip trailing '\','/' symbols (if not preceded by ':') */
-		for(; p > path && IS_PATH_SEPARATOR_W(*p) && p[-1] != L':'; p--) *p = 0;
+		for (; p > path && IS_PATH_SEPARATOR_W(*p) && p[-1] != L':'; p--) *p = 0;
 
 		/* Expand a wildcard in the current file path and store results into data->root_files.
 		 * If a wildcard is not found then just the file path is stored.
@@ -66,7 +66,7 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 		length = p - path + 1;
 		index = wcscspn(path, L"*?");
 
-		if(index < length && wcscspn(path + index, L"/\\") >= (length - index))
+		if (index < length && wcscspn(path + index, L"/\\") >= (length - index))
 		{
 			/* a wildcard is found without a directory separator after it */
 			wchar_t* parent;
@@ -74,17 +74,18 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 			HANDLE handle;
 
 			/* find a directory separator before the file name */
-			for(; index > 0 && !IS_PATH_SEPARATOR(path[index]); index--);
+			for (; index > 0 && !IS_PATH_SEPARATOR(path[index]); index--);
 			parent = (IS_PATH_SEPARATOR(path[index]) ? path : 0);
 
 			handle = FindFirstFileW(path, &d);
-			if(INVALID_HANDLE_VALUE != handle)
+			if (INVALID_HANDLE_VALUE != handle)
 			{
 				do {
 					file_t file;
 					int failed;
 					if (IS_CURRENT_OR_PARENT_DIRW(d.cFileName)) continue;
 
+					memset(&file, 0, sizeof(file));
 					file.wpath = make_pathw(parent, index + 1, d.cFileName);
 					if (!file.wpath) continue;
 
@@ -98,7 +99,7 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 					}
 
 					/* quietly skip unconvertible file names */
-					if(!file.path || failed) {
+					if (!file.path || failed) {
 						free(file.path);
 						free(file.wpath);
 						continue;
@@ -108,7 +109,7 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 					file.mode |= FILE_IFROOT;
 					add_root_file(data, &file);
 					added++;
-				} while(FindNextFileW(handle, &d));
+				} while (FindNextFileW(handle, &d));
 				FindClose(handle);
 			} else {
 				/* report error on the specified wildcard */
@@ -120,23 +121,23 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 		}
 		else
 		{
-			file_t file;
 			int failed;
-			file.wpath = path;
+			file_t file;
+			memset(&file, 0, sizeof(file));
 
 			/* if filepath is a dash string "-" */
 			if ((path[0] == L'-' && path[1] == L'\0'))
 			{
-				file.mtime = file.size = 0;
 				file.mode = FILE_IFSTDIN;
 				file.path = rsh_strdup("(stdin)");
 			} else {
-				file.path = wchar_to_cstr(file.wpath, WIN_DEFAULT_ENCODING, &failed);
+				file.path = wchar_to_cstr(path, WIN_DEFAULT_ENCODING, &failed);
 				if (failed) {
 					log_error(_("Can't convert the path to local encoding: %s\n"), file.path);
 					free(file.path);
 					continue;
 				}
+				file.wpath = path;
 				if (rsh_file_statw(&file) < 0) {
 					log_file_error(file.path);
 					free(file.path);
@@ -152,12 +153,10 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 	} /* for */
 #else
 	/* copy file paths */
-	for(i = 0; i < count; i++)
+	for (i = 0; i < count; i++)
 	{
 		file_t file;
-		file.path = rsh_strdup(paths[i]);
-		file.wpath = 0;
-		file.mtime = file.size = 0;
+		rsh_file_init(&file, paths[i], 0);
 
 		if (IS_DASH_STR(file.path))
 		{
@@ -165,7 +164,7 @@ file_search_data* create_file_search_data(rsh_tchar** paths, size_t count, int m
 		}
 		else if (rsh_file_stat2(&file, USE_LSTAT) < 0) {
 			log_file_error(file.path);
-			free(file.path);
+			rsh_file_cleanup(&file);
 			continue;
 		}
 
@@ -195,21 +194,26 @@ void scan_files(file_search_data* data)
 {
 	size_t i;
 	size_t count = data->root_files.size;
+	int skip_symlink_dirs = !(data->options & FIND_FOLLOW_SYMLINKS);
 
-	for(i = 0; i < count && !(data->options & FIND_CANCEL); i++)
+	for (i = 0; i < count && !(data->options & FIND_CANCEL); i++)
 	{
 		file_t* file = get_root_file(data, i);
 		assert(!!(file->mode & FILE_IFROOT));
 
 		/* check if file is a directory */
-		if(FILE_ISDIR(file)) {
-			if(data->max_depth != 0) {
+		if (FILE_ISDIR(file)) {
+			/* silently skip symlinks to directories if required */
+			if (skip_symlink_dirs && FILE_ISLNK(file)) {
+				continue;
+			}
+
+			if (data->max_depth != 0) {
 				find_file(file, data);
-			} else if((data->options & FIND_LOG_ERRORS) != 0) {
+			} else if ((data->options & FIND_LOG_ERRORS) != 0) {
 				errno = EISDIR;
 				log_file_error(file->path);
 			}
-			continue;
 		} else {
 			/* process a regular file or a dash '-' path */
 			data->call_back(file, data->call_back_data);
@@ -218,7 +222,7 @@ void scan_files(file_search_data* data)
 }
 
 /**
- * A entry of a list containing files from a directory.
+ * An entry of a list containing content of a directory.
  */
 typedef struct dir_entry
 {
@@ -238,10 +242,10 @@ typedef struct dir_entry
 static dir_entry* dir_entry_new(dir_entry *next, char* filename, unsigned type)
 {
 	dir_entry* e = (dir_entry*)malloc(sizeof(dir_entry));
-	if(!e) return NULL;
-	if(filename) {
+	if (!e) return NULL;
+	if (filename) {
 		e->filename = rsh_strdup(filename);
-		if(!e->filename) {
+		if (!e->filename) {
 			free(e);
 			return NULL;
 		}
@@ -264,7 +268,7 @@ static dir_entry* dir_entry_new(dir_entry *next, char* filename, unsigned type)
 static dir_entry* dir_entry_insert(dir_entry **at, char* filename, unsigned type)
 {
 	dir_entry* e = dir_entry_new(*at, filename, type);
-	if(e) *at = e;
+	if (e) *at = e;
 	return e;
 }
 
@@ -286,7 +290,7 @@ static void dir_entry_drop_head(dir_entry** p)
  */
 typedef struct dir_iterator
 {
-	int left;
+	int count;
 	char* dir_path;
 } dir_iterator;
 #define MAX_DIRS_DEPTH 64
@@ -295,49 +299,49 @@ typedef struct dir_iterator
  * Walk directory tree and call given callback function to process each file/directory.
  *
  * @param start_dir path to the directory to walk recursively
- * @param options the options specifying how to walk the directory tree
+ * @param data the options specifying how to walk the directory tree
  * @return 0 on success, -1 on error
  */
-int find_file(file_t* start_dir, file_search_data* options)
+int find_file(file_t* start_dir, file_search_data* data)
 {
 	dir_entry *dirs_stack = NULL; /* root of the dir_list */
 	dir_iterator* it;
 	int level = 0;
-	int max_depth = options->max_depth;
-	int flags = options->options;
+	int max_depth = data->max_depth;
+	int options = data->options;
 	file_t file;
 
-	if(max_depth < 0 || max_depth >= MAX_DIRS_DEPTH) {
+	if (max_depth < 0 || max_depth >= MAX_DIRS_DEPTH) {
 		max_depth = MAX_DIRS_DEPTH - 1;
 	}
 
 	/* skip the directory if max_depth == 0 */
-	if(!max_depth) return 0;
+	if (!max_depth) return 0;
 
-	if(!FILE_ISDIR(start_dir)) {
+	if (!FILE_ISDIR(start_dir)) {
 		errno = ENOTDIR;
 		return -1;
 	}
 
 	/* check if we should descend into the root directory */
-	if((flags & (FIND_WALK_DEPTH_FIRST | FIND_SKIP_DIRS)) == 0) {
-		if(!options->call_back(start_dir, options->call_back_data))
+	if ((options & (FIND_WALK_DEPTH_FIRST | FIND_SKIP_DIRS)) == 0) {
+		if (!data->call_back(start_dir, data->call_back_data))
 			return 0;
 	}
 
 	/* allocate array of counters of directory elements */
 	it = (dir_iterator*)malloc((MAX_DIRS_DEPTH + 1) * sizeof(dir_iterator));
-	if(!it) {
+	if (!it) {
 		return -1;
 	}
 
 	/* push dummy counter for the root element */
-	it[0].left = 1;
+	it[0].count = 1;
 	it[0].dir_path = 0;
 
 	memset(&file, 0, sizeof(file));
 
-	while(!(options->options & FIND_CANCEL))
+	while (!(data->options & FIND_CANCEL))
 	{
 		dir_entry **insert_at;
 		char* dir_path;
@@ -345,21 +349,21 @@ int find_file(file_t* start_dir, file_search_data* options)
 		struct dirent *de;
 
 		/* climb down from the tree */
-		while(--it[level].left < 0) {
+		while (--it[level].count < 0) {
 			/* do not need this dir_path anymore */
 			free(it[level].dir_path);
 
-			if(--level < 0) {
+			if (--level < 0) {
 				/* walked the whole tree */
 				assert(!dirs_stack);
 				free(it);
 				return 0;
 			}
 		}
-		assert(level >= 0 && it[level].left >= 0);
+		assert(level >= 0 && it[level].count >= 0);
 
 		/* take a filename from dirs_stack and construct the next path */
-		if(level) {
+		if (level) {
 			assert(dirs_stack != NULL);
 			dir_path = make_path(it[level].dir_path, dirs_stack->filename);
 			dir_entry_drop_head(&dirs_stack);
@@ -368,63 +372,67 @@ int find_file(file_t* start_dir, file_search_data* options)
 			dir_path = rsh_strdup(start_dir->path);
 		}
 
-		if(!dir_path) continue;
+		if (!dir_path) continue;
 
 		/* fill the next level of directories */
 		level++;
 		assert(level < MAX_DIRS_DEPTH);
-		it[level].left = 0;
+		it[level].count = 0;
 		it[level].dir_path = dir_path;
 
-		if((flags & (FIND_WALK_DEPTH_FIRST | FIND_SKIP_DIRS)) == FIND_WALK_DEPTH_FIRST)
+		if ((options & (FIND_WALK_DEPTH_FIRST | FIND_SKIP_DIRS)) == FIND_WALK_DEPTH_FIRST)
 		{
-			file.path = rsh_strdup(dir_path);
+			rsh_file_init(&file, dir_path, 1);
 			rsh_file_stat2(&file, USE_LSTAT);
 
 			/* check if we should skip the directory */
-			if(!options->call_back(&file, options->call_back_data)) {
+			if (!data->call_back(&file, data->call_back_data)) {
 				rsh_file_cleanup(&file);
 				continue;
 			}
 		}
+		rsh_file_cleanup(&file);
 
 		/* step into directory */
 		dp = opendir(dir_path);
-		if(!dp) continue;
+		if (!dp) continue;
 
 		insert_at = &dirs_stack;
 
-		while((de = readdir(dp)) != NULL)
+		while ((de = readdir(dp)) != NULL)
 		{
 			int res;
 
 			/* skip the "." and ".." directories */
-			if(IS_CURRENT_OR_PARENT_DIR(de->d_name)) continue;
+			if (IS_CURRENT_OR_PARENT_DIR(de->d_name)) continue;
 
+			file.mode = 0;
 			file.path = make_path(dir_path, de->d_name);
-			if(!file.path) continue;
+			if (!file.path) continue;
 
 			res  = rsh_file_stat2(&file, USE_LSTAT);
-			if(res >= 0) {
+			if (res >= 0) {
 				/* process the file or directory */
-				if(FILE_ISDIR(&file) && (flags & (FIND_WALK_DEPTH_FIRST | FIND_SKIP_DIRS))) {
-					res = 1;
-				} else {
+				if (FILE_ISDIR(&file) && (options & (FIND_WALK_DEPTH_FIRST | FIND_SKIP_DIRS))) {
+					res = ((options & FIND_FOLLOW_SYMLINKS) || !FILE_ISLNK(&file));
+				} else if (FILE_ISREG(&file)) {
 					/* handle file by callback function */
-					res = options->call_back(&file, options->call_back_data);
+					res = data->call_back(&file, data->call_back_data);
 				}
 
 				/* check if file is a directory and we need to walk it, */
 				/* but don't go deeper than max_depth */
-				if(FILE_ISDIR(&file) && res && level < max_depth) {
+				if (FILE_ISDIR(&file) && res && level < max_depth &&
+					((options & FIND_FOLLOW_SYMLINKS) || !FILE_ISLNK(&file)))
+				{
 					/* add the directory name to the dirs_stack */
-					if(dir_entry_insert(insert_at, de->d_name, file.mode)) {
+					if (dir_entry_insert(insert_at, de->d_name, file.mode)) {
 						/* the directory name was successfully inserted */
 						insert_at = &((*insert_at)->next);
-						it[level].left++;
+						it[level].count++;
 					}
 				}
-			} else if (options->options & FIND_LOG_ERRORS) {
+			} else if (options & FIND_LOG_ERRORS) {
 				/* report error only if FIND_LOG_ERRORS option is set */
 				log_file_error(file.path);
 			}
@@ -440,6 +448,6 @@ int find_file(file_t* start_dir, file_search_data* options)
 		free(it[level--].dir_path);
 	}
 	free(it);
-	rsh_file_cleanup(&file);
+	assert(file.path == 0);
 	return 0;
 }
