@@ -10,9 +10,9 @@ extern "C" {
 /* algorithm context */
 typedef struct ed2k_ctx
 {
-	md4_ctx md4_context;       /* context to hash block hashes */
-	md4_ctx md4_context_inner; /* context to hash file blocks */
-	int not_emule;
+	md4_ctx md4_context_inner; /* md4 context to hash file blocks */
+	md4_ctx md4_context;       /* md4 context to hash block hashes */
+	int not_emule;             /* flag: 0 for emule ed2k algorithm */
 } ed2k_ctx;
 
 /* hash functions */
