@@ -91,7 +91,7 @@ RHASH_API rhash rhash_init(unsigned hash_id)
 	char* phash_ctx;
 
 	hash_id &= RHASH_ALL_HASHES;
-	if(hash_id == 0) {
+	if (hash_id == 0) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -101,18 +101,18 @@ RHASH_API rhash rhash_init(unsigned hash_id)
 
 	id = 1 << tail_bit_index;
 
-	if(hash_id == id) {
+	if (hash_id == id) {
 		/* handle the most common case of only one hash */
 		num = 1;
 		info = &rhash_info_table[tail_bit_index];
 		hash_size_sum = info->context_size;
 	} else {
 		/* another case: hash_id contains several hashes */
-		for(bit_index = tail_bit_index; id <= hash_id; bit_index++, id = id << 1) {
+		for (bit_index = tail_bit_index; id <= hash_id; bit_index++, id = id << 1) {
 			assert(id != 0);
 			assert(bit_index < RHASH_HASH_COUNT);
 			info = &rhash_info_table[bit_index];
-			if(hash_id & id) {
+			if (hash_id & id) {
 				/* align sizes by 8 bytes */
 				aligned_size = (info->context_size + 7) & ~7;
 				hash_size_sum += aligned_size;
@@ -128,7 +128,7 @@ RHASH_API rhash rhash_init(unsigned hash_id)
 
 	/* allocate rhash context with enough memory to store contexts of all used hashes */
 	rctx = (rhash_context_ext*)malloc(aligned_size + hash_size_sum);
-	if(rctx == NULL) return NULL;
+	if (rctx == NULL) return NULL;
 
 	/* initialize common fields of the rhash context */
 	memset(rctx, 0, sizeof(rhash_context_ext));
@@ -142,11 +142,11 @@ RHASH_API rhash rhash_init(unsigned hash_id)
 	assert(phash_ctx >= (char*)&rctx->vector[num]);
 
 	/* initialize context for every hash in a loop */
-	for(bit_index = tail_bit_index, id = 1 << tail_bit_index, i = 0;
+	for (bit_index = tail_bit_index, id = 1 << tail_bit_index, i = 0;
 		id <= hash_id; bit_index++, id = id << 1)
 	{
 		/* check if a hash function with given id shall be included into rctx */
-		if((hash_id & id) != 0) {
+		if ((hash_id & id) != 0) {
 			info = &rhash_info_table[bit_index];
 			assert(info->context_size > 0);
 			assert(((phash_ctx - (char*)0) & 7) == 0); /* hash context is aligned */
@@ -156,7 +156,7 @@ RHASH_API rhash rhash_init(unsigned hash_id)
 			rctx->vector[i].context = phash_ctx;
 
 			/* BTIH initialization is complex, save pointer for later */
-			if((id & RHASH_BTIH) != 0) rctx->bt_ctx = phash_ctx;
+			if ((id & RHASH_BTIH) != 0) rctx->bt_ctx = phash_ctx;
 			phash_ctx += (info->context_size + 7) & ~7;
 
 			/* initialize the i-th hash context */
@@ -178,14 +178,14 @@ void rhash_free(rhash ctx)
 	rhash_context_ext* const ectx = (rhash_context_ext*)ctx;
 	unsigned i;
 
-	if(ctx == 0) return;
+	if (ctx == 0) return;
 	assert(ectx->hash_vector_size <= RHASH_HASH_COUNT);
 	ectx->state = STATE_DELETED; /* mark memory block as being removed */
 
 	/* clean the hash functions, which require additional clean up */
-	for(i = 0; i < ectx->hash_vector_size; i++) {
+	for (i = 0; i < ectx->hash_vector_size; i++) {
 		struct rhash_hash_info* info = ectx->vector[i].hash_info;
-		if(info->cleanup != 0) {
+		if (info->cleanup != 0) {
 			info->cleanup(ectx->vector[i].context);
 		}
 	}
@@ -209,9 +209,9 @@ RHASH_API void rhash_reset(rhash ctx)
 	ectx->state = STATE_ACTIVE; /* re-activate the structure */
 
 	/* re-initialize every hash in a loop */
-	for(i = 0; i < ectx->hash_vector_size; i++) {
+	for (i = 0; i < ectx->hash_vector_size; i++) {
 		struct rhash_hash_info* info = ectx->vector[i].hash_info;
-		if(info->cleanup != 0) {
+		if (info->cleanup != 0) {
 			info->cleanup(ectx->vector[i].context);
 		}
 
@@ -236,12 +236,12 @@ RHASH_API int rhash_update(rhash ctx, const void* message, size_t length)
 	unsigned i;
 
 	assert(ectx->hash_vector_size <= RHASH_HASH_COUNT);
-	if(ectx->state != STATE_ACTIVE) return 0; /* do nothing if canceled */
+	if (ectx->state != STATE_ACTIVE) return 0; /* do nothing if canceled */
 
 	ctx->msg_size += length;
 
 	/* call update method for every algorithm */
-	for(i = 0; i < ectx->hash_vector_size; i++) {
+	for (i = 0; i < ectx->hash_vector_size; i++) {
 		struct rhash_hash_info* info = ectx->vector[i].hash_info;
 		assert(info->update != 0);
 		info->update(ectx->vector[i].context, message, length);
@@ -265,11 +265,11 @@ RHASH_API int rhash_final(rhash ctx, unsigned char* first_result)
 	assert(ectx->hash_vector_size <= RHASH_HASH_COUNT);
 
 	/* skip final call if already finalized and auto-final is on */
-	if((ectx->flags & RCTX_FINALIZED_MASK) ==
+	if ((ectx->flags & RCTX_FINALIZED_MASK) ==
 		(RCTX_AUTO_FINAL | RCTX_FINALIZED)) return 0;
 
 	/* call final method for every algorithm */
-	for(i = 0; i < ectx->hash_vector_size; i++) {
+	for (i = 0; i < ectx->hash_vector_size; i++) {
 		struct rhash_hash_info* info = ectx->vector[i].hash_info;
 		assert(info->final != 0);
 		assert(info->info->digest_size < sizeof(buffer));
@@ -301,29 +301,29 @@ static void rhash_put_digest(rhash ctx, unsigned hash_id, unsigned char* result)
 	assert(ectx->hash_vector_size > 0 && ectx->hash_vector_size <= RHASH_HASH_COUNT);
 
 	/* finalize context if not yet finalized and auto-final is on */
-	if((ectx->flags & RCTX_FINALIZED_MASK) == RCTX_AUTO_FINAL) {
+	if ((ectx->flags & RCTX_FINALIZED_MASK) == RCTX_AUTO_FINAL) {
 		rhash_final(ctx, NULL);
 	}
 
-	if(hash_id == 0) {
+	if (hash_id == 0) {
 		item = &ectx->vector[0]; /* get the first hash */
 		info = item->hash_info;
 	} else {
-		for(i = 0;; i++) {
-			if(i >= ectx->hash_vector_size) {
+		for (i = 0;; i++) {
+			if (i >= ectx->hash_vector_size) {
 				return; /* hash_id not found, do nothing */
 			}
 			item = &ectx->vector[i];
 			info = item->hash_info;
-			if(info->info->hash_id == hash_id) break;
+			if (info->info->hash_id == hash_id) break;
 		}
 	}
 	digest = ((unsigned char*)item->context + info->digest_diff);
-	if(info->info->flags & F_SWAP32) {
+	if (info->info->flags & F_SWAP32) {
 		assert((info->info->digest_size & 3) == 0);
 		/* NB: the next call is correct only for multiple of 4 byte size */
 		rhash_swap_copy_str_to_u32(result, 0, digest, info->info->digest_size);
-	} else if(info->info->flags & F_SWAP64) {
+	} else if (info->info->flags & F_SWAP64) {
 		rhash_swap_copy_u64_to_str(result, digest, info->info->digest_size);
 	} else {
 		memcpy(result, digest, info->info->digest_size);
@@ -363,7 +363,7 @@ RHASH_API int rhash_msg(unsigned hash_id, const void* message, size_t length, un
 	rhash ctx;
 	hash_id &= RHASH_ALL_HASHES;
 	ctx = rhash_init(hash_id);
-	if(ctx == NULL) return -1;
+	if (ctx == NULL) return -1;
 	rhash_update(ctx, message, length);
 	rhash_final(ctx, result);
 	rhash_free(ctx);
@@ -388,31 +388,32 @@ RHASH_API int rhash_file_update(rhash ctx, FILE* fd)
 	unsigned char *buffer, *pmem;
 	size_t length = 0, align8;
 	int res = 0;
-	if(ectx->state != STATE_ACTIVE) return 0; /* do nothing if canceled */
+	if (ectx->state != STATE_ACTIVE) return 0; /* do nothing if canceled */
 
-	if(ctx == NULL) {
+	if (ctx == NULL) {
 		errno = EINVAL;
 		return -1;
 	}
 
 	pmem = (unsigned char*)malloc(block_size + 8);
-	if(!pmem) return -1; /* errno is set to ENOMEM according to UNIX 98 */
+	if (!pmem) return -1; /* errno is set to ENOMEM according to UNIX 98 */
 
 	align8 = ((unsigned char*)0 - pmem) & 7;
 	buffer = pmem + align8;
 
-	while(!feof(fd)) {
-		if(ectx->state != STATE_ACTIVE) break; /* stop if canceled */
+	while (!feof(fd)) {
+		/* stop if canceled */
+		if (ectx->state != STATE_ACTIVE) break;
 
 		length = fread(buffer, 1, block_size, fd);
-		/* read can return -1 on error */
-		if(length == (size_t)-1) {
-			res = -1; /* note: fread sets errno */
+
+		if (ferror(fd)) {
+			res = -1; /* note: errno contains error code */
 			break;
 		} else if (length) {
 			rhash_update(ctx, buffer, length);
 
-			if(ectx->callback) {
+			if (ectx->callback) {
 				((rhash_callback_t)ectx->callback)(ectx->callback_data, ectx->rc.msg_size);
 			}
 		}
@@ -437,14 +438,14 @@ RHASH_API int rhash_file(unsigned hash_id, const char* filepath, unsigned char*
 	int res;
 
 	hash_id &= RHASH_ALL_HASHES;
-	if(hash_id == 0) {
+	if (hash_id == 0) {
 		errno = EINVAL;
 		return -1;
 	}
 
-	if((fd = fopen(filepath, "rb")) == NULL) return -1;
+	if ((fd = fopen(filepath, "rb")) == NULL) return -1;
 
-	if((ctx = rhash_init(hash_id)) == NULL) return -1;
+	if ((ctx = rhash_init(hash_id)) == NULL) return -1;
 
 	res = rhash_file_update(ctx, fd); /* hash the file */
 	fclose(fd);
@@ -472,14 +473,14 @@ RHASH_API int rhash_wfile(unsigned hash_id, const wchar_t* filepath, unsigned ch
 	int res;
 
 	hash_id &= RHASH_ALL_HASHES;
-	if(hash_id == 0) {
+	if (hash_id == 0) {
 		errno = EINVAL;
 		return -1;
 	}
 
-	if((fd = _wfsopen(filepath, L"rb", _SH_DENYWR)) == NULL) return -1;
+	if ((fd = _wfsopen(filepath, L"rb", _SH_DENYWR)) == NULL) return -1;
 
-	if((ctx = rhash_init(hash_id)) == NULL) return -1;
+	if ((ctx = rhash_init(hash_id)) == NULL) return -1;
 
 	res = rhash_file_update(ctx, fd); /* hash the file */
 	fclose(fd);
@@ -502,7 +503,7 @@ const rhash_info* rhash_info_by_id(unsigned hash_id)
 {
 	hash_id &= RHASH_ALL_HASHES;
 	/* check that only one bit is set */
-	if(hash_id != (hash_id & -(int)hash_id)) return NULL;
+	if (hash_id != (hash_id & -(int)hash_id)) return NULL;
 	/* note: alternative condition is (hash_id == 0 || (hash_id & (hash_id - 1)) != 0) */
 	return rhash_info_table[rhash_ctz(hash_id)].info;
 }
@@ -528,7 +529,7 @@ RHASH_API int rhash_is_base32(unsigned hash_id)
 RHASH_API int rhash_get_digest_size(unsigned hash_id)
 {
 	hash_id &= RHASH_ALL_HASHES;
-	if(hash_id == 0 || (hash_id & (hash_id - 1)) != 0) return -1;
+	if (hash_id == 0 || (hash_id & (hash_id - 1)) != 0) return -1;
 	return (int)rhash_info_table[rhash_ctz(hash_id)].info->digest_size;
 }
 
@@ -578,29 +579,29 @@ static size_t rhash_get_magnet_url_size(const char* filepath,
 	unsigned bit, hash = context->hash_id & hash_mask;
 
 	/* RHPR_NO_MAGNET, RHPR_FILESIZE */
-	if((flags & RHPR_NO_MAGNET) == 0) {
+	if ((flags & RHPR_NO_MAGNET) == 0) {
 		size += 8;
 	}
 
-	if((flags & RHPR_FILESIZE) != 0) {
+	if ((flags & RHPR_FILESIZE) != 0) {
 		uint64_t num = context->msg_size;
 
 		size += 4;
-		if(num == 0) size++;
+		if (num == 0) size++;
 		else {
-			for(; num; num /= 10, size++);
+			for (; num; num /= 10, size++);
 		}
 	}
 
-	if(filepath) {
+	if (filepath) {
 		size += 4 + rhash_urlencode(NULL, filepath);
 	}
 
 	/* loop through hash values */
-	for(bit = hash & -(int)hash; bit <= hash; bit <<= 1) {
+	for (bit = hash & -(int)hash; bit <= hash; bit <<= 1) {
 		const char* name;
-		if((bit & hash) == 0) continue;
-		if((name = rhash_get_magnet_name(bit)) == 0) continue;
+		if ((bit & hash) == 0) continue;
+		if ((name = rhash_get_magnet_name(bit)) == 0) continue;
 
 		size += (7 + 2) + strlen(name);
 		size += rhash_print(NULL, context, bit,
@@ -630,23 +631,23 @@ RHASH_API size_t rhash_print_magnet(char* output, const char* filepath,
 	int i;
 	const char* begin = output;
 
-	if(output == NULL) return rhash_get_magnet_url_size(
+	if (output == NULL) return rhash_get_magnet_url_size(
 		filepath, context, hash_mask, flags);
 
 	/* RHPR_NO_MAGNET, RHPR_FILESIZE */
-	if((flags & RHPR_NO_MAGNET) == 0) {
+	if ((flags & RHPR_NO_MAGNET) == 0) {
 		strcpy(output, "magnet:?");
 		output += 8;
 	}
 
-	if((flags & RHPR_FILESIZE) != 0) {
+	if ((flags & RHPR_FILESIZE) != 0) {
 		strcpy(output, "xl=");
 		output += 3;
 		output += rhash_sprintI64(output, context->msg_size);
 		*(output++) = '&';
 	}
 
-	if(filepath) {
+	if (filepath) {
 		strcpy(output, "dn=");
 		output += 3;
 		output += rhash_urlencode(output, filepath);
@@ -654,18 +655,18 @@ RHASH_API size_t rhash_print_magnet(char* output, const char* filepath,
 	}
 	flags &= RHPR_UPPERCASE;
 
-	for(i = 0; i < 2; i++) {
+	for (i = 0; i < 2; i++) {
 		unsigned bit;
 		unsigned hash = context->hash_id & hash_mask;
 		hash = (i == 0 ? hash & (RHASH_ED2K | RHASH_AICH)
 			: hash & ~(RHASH_ED2K | RHASH_AICH));
-		if(!hash) continue;
+		if (!hash) continue;
 
 		/* loop through hash values */
-		for(bit = hash & -(int)hash; bit <= hash; bit <<= 1) {
+		for (bit = hash & -(int)hash; bit <= hash; bit <<= 1) {
 			const char* name;
-			if((bit & hash) == 0) continue;
-			if(!(name = rhash_get_magnet_name(bit))) continue;
+			if ((bit & hash) == 0) continue;
+			if (!(name = rhash_get_magnet_name(bit))) continue;
 
 			strcpy(output, "xt=urn:");
 			output += 7;
@@ -702,7 +703,7 @@ size_t rhash_print_bytes(char* output, const unsigned char* bytes,
 	int upper_case = (flags & RHPR_UPPERCASE);
 	int format = (flags & ~RHPR_MODIFIER);
 
-	switch(format) {
+	switch (format) {
 	case RHPR_HEX:
 		str_len = size * 2;
 		rhash_byte_to_hex(output, bytes, (unsigned)size, upper_case);
@@ -747,18 +748,18 @@ size_t RHASH_API rhash_print(char* output, rhash context, unsigned hash_id, int
 	info = (hash_id != 0 ? rhash_info_by_id(hash_id) :
 		((rhash_context_ext*)context)->vector[0].hash_info->info);
 
-	if(info == NULL) return 0;
+	if (info == NULL) return 0;
 	digest_size = info->digest_size;
 	assert(digest_size <= 64);
 
 	flags &= (RHPR_FORMAT | RHPR_MODIFIER);
-	if((flags & RHPR_FORMAT) == 0) {
+	if ((flags & RHPR_FORMAT) == 0) {
 		/* use default format if not specified by flags */
 		flags |= (info->flags & RHASH_INFO_BASE32 ? RHPR_BASE32 : RHPR_HEX);
 	}
 
-	if(output == NULL) {
-		switch(flags & RHPR_FORMAT) {
+	if (output == NULL) {
+		switch (flags & RHPR_FORMAT) {
 		case RHPR_HEX:
 			return (digest_size * 2);
 		case RHPR_BASE32:
@@ -773,11 +774,11 @@ size_t RHASH_API rhash_print(char* output, rhash context, unsigned hash_id, int
 	/* note: use info->hash_id, cause hash_id can be 0 */
 	rhash_put_digest(context, info->hash_id, digest);
 
-	if((flags & ~RHPR_UPPERCASE) == (RHPR_REVERSE | RHPR_HEX)) {
+	if ((flags & ~RHPR_UPPERCASE) == (RHPR_REVERSE | RHPR_HEX)) {
 		/* reverse the digest */
 		unsigned char *p = digest, *r = digest + digest_size - 1;
 		char tmp;
-		for(; p < r; p++, r--) {
+		for (; p < r; p++, r--) {
 			tmp = *p;
 			*p = *r;
 			*r = tmp;
@@ -819,9 +820,9 @@ BOOL APIENTRY DllMain(HMODULE hModule, DWORD reason, LPVOID reserved)
  */
 static rhash_uptr_t process_bt_msg(unsigned msg_id, torrent_ctx* bt, rhash_uptr_t ldata, rhash_uptr_t rdata)
 {
-	if(bt == NULL) return RHASH_ERROR;
+	if (bt == NULL) return RHASH_ERROR;
 
-	switch(msg_id) {
+	switch (msg_id) {
 	case RMSG_BT_ADD_FILE:
 		bt_add_file(bt, (const char*)ldata, *(unsigned long long*)rdata);
 		break;
@@ -829,7 +830,7 @@ static rhash_uptr_t process_bt_msg(unsigned msg_id, torrent_ctx* bt, rhash_uptr_
 		bt_set_options(bt, (unsigned)ldata);
 		break;
 	case RMSG_BT_SET_ANNOUNCE:
-		bt_set_announce(bt, (const char*)ldata);
+		bt_add_announce(bt, (const char*)ldata);
 		break;
 	case RMSG_BT_SET_PIECE_LENGTH:
 		bt_set_piece_length(bt, (size_t)ldata);
@@ -842,7 +843,7 @@ static rhash_uptr_t process_bt_msg(unsigned msg_id, torrent_ctx* bt, rhash_uptr_
 		bt_set_program_name(bt, (const char*)ldata);
 		break;
 	case RMSG_BT_GET_TEXT:
-		return RHASH_STR2UPTR(bt_get_text(bt, (char**)ldata));
+		return (rhash_uptr_t)bt_get_text(bt, (char**)ldata);
 	default:
 		return RHASH_ERROR; /* unknown message */
 	}
@@ -865,13 +866,13 @@ RHASH_API rhash_uptr_t rhash_transmit(unsigned msg_id, void* dst, rhash_uptr_t l
 	/* for messages working with rhash context */
 	rhash_context_ext* const ctx = (rhash_context_ext*)dst;
 
-	switch(msg_id) {
+	switch (msg_id) {
 	case RMSG_GET_CONTEXT:
 		{
 			unsigned i;
-			for(i = 0; i < ctx->hash_vector_size; i++) {
+			for (i = 0; i < ctx->hash_vector_size; i++) {
 				struct rhash_hash_info* info = ctx->vector[i].hash_info;
-				if(info->info->hash_id == (unsigned)ldata)
+				if (info->info->hash_id == (unsigned)ldata)
 					return PVOID2UPTR(ctx->vector[i].context);
 			}
 			return (rhash_uptr_t)0;
@@ -889,7 +890,7 @@ RHASH_API rhash_uptr_t rhash_transmit(unsigned msg_id, void* dst, rhash_uptr_t l
 		return ((ctx->flags & RCTX_FINALIZED) != 0);
 	case RMSG_SET_AUTOFINAL:
 		ctx->flags &= ~RCTX_AUTO_FINAL;
-		if(ldata) ctx->flags |= RCTX_AUTO_FINAL;
+		if (ldata) ctx->flags |= RCTX_AUTO_FINAL;
 		break;
 
 	/* OpenSSL related messages */
