@@ -32,18 +32,19 @@ enum {
 	MODE_TORRENT   = 0x10,
 
 	/* misc options */
-	OPT_EMBED_CRC = 0x20,
-	OPT_RECURSIVE = 0x40,
-	OPT_SKIP_OK   = 0x80,
-	OPT_IGNORE_CASE = 0x100,
-	OPT_VERBOSE   = 0x200,
-	OPT_PERCENTS  = 0x400,
-	OPT_SPEED     = 0x800,
-	OPT_BT_PRIVATE  = 0x1000,
-	OPT_UPPERCASE = 0x2000,
-	OPT_LOWERCASE = 0x4000,
-	OPT_GOST_REVERSE = 0x8000,
-	OPT_BENCH_RAW = 0x10000,
+	OPT_EMBED_CRC  = 0x20,
+	OPT_RECURSIVE  = 0x40,
+	OPT_FOLLOW     = 0x80,
+	OPT_SKIP_OK    = 0x100,
+	OPT_IGNORE_CASE = 0x200,
+	OPT_VERBOSE    = 0x400,
+	OPT_PERCENTS   = 0x800,
+	OPT_SPEED      = 0x1000,
+	OPT_BT_PRIVATE = 0x2000,
+	OPT_UPPERCASE  = 0x4000,
+	OPT_LOWERCASE  = 0x8000,
+	OPT_GOST_REVERSE = 0x10000,
+	OPT_BENCH_RAW  = 0x20000,
 
 #ifdef _WIN32
 	OPT_UTF8 = 0x10000000,
@@ -70,6 +71,7 @@ struct options_t
 	unsigned sum_flags;  /* flags to specify what sums will be calculated */
 	unsigned fmt;        /* flags to specify output format to use */
 	unsigned mode;       /* flags to specify program mode */
+	unsigned openssl_mask;  /* bit-mask for enabled OpenSSL hash functions */
 	const char* config_file; /* config file path */
 	char* printf_str;        /* printf-like format */
 	char* template_file; /* printf-like template file path */
@@ -78,17 +80,17 @@ struct options_t
 	char* embed_crc_delimiter;
 	char  path_separator;
 	int   find_max_depth;
-	struct vector_t *files_accept; /* suffixes of files for which sums will be calculated */
+	struct vector_t *files_accept; /* suffixes of files to process */
+	struct vector_t *files_exclude; /* suffixes of files to exclude from processing */
 	struct vector_t *crc_accept;   /* suffixes of crc files to verify or update */
-	unsigned openssl_mask;  /* mask which OpenSSL hashes to use */
+	struct vector_t * bt_announce; /* BitTorrent announce URL */
 	size_t bt_piece_length; /* BitTorrent piece length */
-	char*  bt_announce;     /* BitTorrent announce URL */
-	char*  bt_batch_file;   /* path to save batch torrent to */
+	char*  bt_batch_file;   /* path to save a batch torrent to */
 
 	char** argv;
 	int n_files; /* the number of files obtained from the command line */
 	struct file_search_data* search_data; /* files obtained from the command line */
-	struct vector_t *mem; /* heap variables that should be freed */
+	struct vector_t *mem; /* allocated memory blocks that must be freed on exit */
 };
 extern struct options_t opt;
 
