@@ -15,7 +15,6 @@
 #include <assert.h>
 
 #include "librhash/rhash.h"
-#include "librhash/rhash_timing.h"
 #include "win_utils.h"
 #include "find_file.h"
 #include "calc_sums.h"
@@ -24,7 +23,6 @@
 #include "hash_print.h"
 #include "parse_cmdline.h"
 #include "output.h"
-#include "version.h"
 
 #include "rhash_main.h"
 
@@ -66,7 +64,9 @@ static int find_file_callback(file_t* file, int preprocess)
 	}
 
 	if (preprocess) {
-		if (!file_mask_match(opt.files_accept, file->path) || must_skip_file(file)) {
+		if (!file_mask_match(opt.files_accept, file->path) ||
+			(opt.files_exclude && file_mask_match(opt.files_exclude, file->path)) ||
+			must_skip_file(file)) {
 			return 0;
 		}
 
@@ -79,10 +79,17 @@ static int find_file_callback(file_t* file, int preprocess)
 		int not_root = !(file->mode & FILE_IFROOT);
 
 		if (not_root) {
-			/* only check and update modes use the crc_accept mask */
-			file_mask_array* masks = (opt.mode & (MODE_CHECK | MODE_UPDATE) ?
-				opt.crc_accept : opt.files_accept);
-			if (!file_mask_match(masks, file->path)) return 0;
+			if ((opt.mode & (MODE_CHECK | MODE_UPDATE)) != 0) {
+				/* check and update modes use the crc_accept list */
+				if (!file_mask_match(opt.crc_accept, file->path)) {
+					return 0;
+				}
+			} else {
+				if (!file_mask_match(opt.files_accept, file->path) ||
+					(opt.files_exclude && file_mask_match(opt.files_exclude, file->path))) {
+					return 0;
+				}
+			}
 		}
 		if (must_skip_file(file)) return 0;
 
@@ -144,8 +151,8 @@ static int load_printf_template(void)
 
 	while (!feof(fd)) {
 		len = fread(buffer, 1, 8192, fd);
-		/* read can return -1 on error */
-		if (len == (size_t)-1) break;
+		if (ferror(fd)) break;
+
 		rsh_str_append_n(rhash_data.template_text, buffer, len);
 		if (rhash_data.template_text->len >= MAX_TEMPLATE_SIZE) {
 			log_msg(_("%s: template file is too big\n"), opt.template_file);
@@ -212,17 +219,18 @@ int main(int argc, char *argv[])
 	rhash_library_init();
 
 	/* in benchmark mode just run benchmark and exit */
-	if(opt.mode & MODE_BENCHMARK) {
-		unsigned flags = (opt.flags & OPT_BENCH_RAW ? RHASH_BENCHMARK_CPB | RHASH_BENCHMARK_RAW : RHASH_BENCHMARK_CPB);
-		if((opt.flags & OPT_BENCH_RAW) == 0) {
-			fprintf(rhash_data.out, _("%s v%s benchmarking...\n"), PROGRAM_NAME, VERSION);
+	if (opt.mode & MODE_BENCHMARK) {
+		unsigned flags = (opt.flags & OPT_BENCH_RAW ? BENCHMARK_CPB | BENCHMARK_RAW : BENCHMARK_CPB);
+		if ((opt.flags & OPT_BENCH_RAW) == 0) {
+			fprintf(rhash_data.out, _("%s v%s benchmarking...\n"), PROGRAM_NAME, get_version_string());
 		}
-		rhash_run_benchmark(opt.sum_flags, flags, rhash_data.out);
-		rsh_exit(0);
+		run_benchmark(opt.sum_flags, flags);
+		exit_code = (rhash_data.interrupted ? 3 : 0);
+		rsh_exit(exit_code);
 	}
 
-	if(opt.n_files == 0) {
-		if(argc > 1) {
+	if (opt.n_files == 0) {
+		if (argc > 1) {
 			log_warning(_("no files/directories were specified at command line\n"));
 		}
 
@@ -236,33 +244,34 @@ int main(int argc, char *argv[])
 	/* setup printf formating string */
 	rhash_data.printf_str = opt.printf_str;
 
-	if(opt.template_file) {
-		if(!load_printf_template()) rsh_exit(2);
-	} else if(!rhash_data.printf_str && !(opt.mode & (MODE_CHECK | MODE_CHECK_EMBEDDED))) {
+	if (opt.template_file) {
+		if (!load_printf_template()) rsh_exit(2);
+	} else if (!rhash_data.printf_str && !(opt.mode & (MODE_CHECK | MODE_CHECK_EMBEDDED))) {
 		/* initialize printf output format according to '--<hashname>' options */
 		init_printf_format( (rhash_data.template_text = rsh_str_new()) );
 		rhash_data.printf_str = rhash_data.template_text->str;
 
-		if(opt.flags & OPT_VERBOSE) {
+		if (opt.flags & OPT_VERBOSE) {
 			char* str = rsh_strdup(rhash_data.printf_str);
 			log_msg(_("Format string is: %s\n"), str_trim(str));
 			free(str);
 		}
 	}
 
-	if(rhash_data.printf_str) {
+	if (rhash_data.printf_str) {
 		rhash_data.print_list = parse_print_string(rhash_data.printf_str, &opt.sum_flags);
 	}
 
 	opt.search_data->options = FIND_SKIP_DIRS;
+	opt.search_data->options |= (opt.flags & OPT_FOLLOW ? FIND_FOLLOW_SYMLINKS : 0);
 	opt.search_data->call_back = find_file_callback;
 
-	if((sfv = (opt.fmt == FMT_SFV && !opt.mode))) {
+	if ((sfv = (opt.fmt == FMT_SFV && !opt.mode))) {
 		print_sfv_banner(rhash_data.out);
 	}
 
 	/* preprocess files */
-	if(sfv || opt.bt_batch_file) {
+	if (sfv || opt.bt_batch_file) {
 		/* note: errors are not reported on preprocessing */
 		opt.search_data->call_back_data = 1;
 		scan_files(opt.search_data);
@@ -271,35 +280,35 @@ int main(int argc, char *argv[])
 	}
 
 	/* measure total processing time */
-	rhash_timer_start(&timer);
+	rsh_timer_start(&timer);
 	rhash_data.processed = 0;
 
 	/* process files */
-	opt.search_data->options = FIND_SKIP_DIRS | FIND_LOG_ERRORS;
+	opt.search_data->options |= FIND_LOG_ERRORS;
 	opt.search_data->call_back_data = 0;
 	scan_files(opt.search_data);
 
-	if((opt.mode & MODE_CHECK_EMBEDDED) && rhash_data.processed > 1) {
+	if ((opt.mode & MODE_CHECK_EMBEDDED) && rhash_data.processed > 1) {
 		print_check_stats();
 	}
 
-	if(!rhash_data.interrupted)
+	if (!rhash_data.interrupted)
 	{
-		if(opt.bt_batch_file && rhash_data.rctx) {
+		if (opt.bt_batch_file && rhash_data.rctx) {
 			rhash_final(rhash_data.rctx, 0);
 			save_torrent_to(opt.bt_batch_file, rhash_data.rctx);
 		}
 
-		if((opt.flags & OPT_SPEED) &&
+		if ((opt.flags & OPT_SPEED) &&
 			!(opt.mode & (MODE_CHECK | MODE_UPDATE)) &&
 			rhash_data.processed > 1)
 		{
-			double time = rhash_timer_stop(&timer);
+			double time = rsh_timer_stop(&timer);
 			print_time_stats(time, rhash_data.total_size, 1);
 		}
 	} else {
 		/* check if interruption was not reported yet */
-		if(rhash_data.interrupted == 1) report_interrupted();
+		if (rhash_data.interrupted == 1) report_interrupted();
 	}
 
 	exit_code = (rhash_data.error_flag ? 1 :
